==> maxdb-source-7_6_00_09.tgz: V76_00_09/MaxDB_ORG/sys/src/SAPDB/DBM/Srv/Commands/Dbm/Shm/DBMSrvCmdDbmShm_DbmShmInfo.cpp <==
/*!
  \file    DBMSrvCmdDbmShm_DbmShmInfo.cpp
  \author  MarcW
  \ingroup DBMServer shared memory administration commands
  \brief   list information about shared memory

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (C) 2002-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/
#include "hcn20.h"

#include "RunTime/System/RTESys_ProcessState.hpp"
#include "DBM/Srv/Commands/Dbm/Shm/DBMSrvCmdDbmShm_DbmShmInfo.hpp"
#include "DBM/Srv/SharedMemory/DBMSrvShM_ShmAdmin.hpp"
#include "DBM/Srv/Message/DBMSrvMsg_Error.hpp"

DBMSrvCmdDbmShm_DbmShmInfo::DBMSrvCmdDbmShm_DbmShmInfo()
: DBMSrv_Command( DBMSrv_Command::KeyDbmShmInfo, false )
{
    m_longHelp=
        "@command dbm_shm_info This command lists information about the DBM shared "
            "memory: version, locking status, connected processes and information about "
            "the persistence if necessary."
        "@preconditions You have the server authorization SharedMemoryAdmin."
        "@syntax dbm_shm_info"
        "@reply OK<NL>"
            "VERSION<NL>"
            "Required <req_version><NL>"
            "Found    <curr_version><NL>"
            "<NL>"
            "LOCKING<NL>"
            "Normal lock  <status_normal_lock><NL>"
            "Backing lock <status_backing_lock><NL>"
            "Locking pid <locking_pid><NL>"
            "<NL>"
            "PROCESSES<NL>"
            "pid | status<NL>"
            "<pid_1> | <status_1><NL>"
            "...<NL>"
            "<pid_n> | <status_n><NL>"
            "<NL>"
            "[PERSISTENCE<NL>"
            "<persistence_error>]"
        "@replyvalue <pid_i> process id of a connected process"
        "@replyvalue <status_i> status of process <pid_i>. Status can be \"ALIVE\", \"DEAD\" "
            "or \"UNKNOWN\""
        "@replyvalue <req_version> required version of shared memory in the format "
            "nn.nn.nn.nnn"
        "@replyvalue <curr_version> version of shared memory found on disk in the format "
            "nn.nn.nn.nnn"
        "@replyvalue <status_normal_lock> status of shared memory lock. Can be \"LOCKED\" "
            "or \"UNLOCKED\""
        "@replyvalue <status_backing_lock> status of backing shared memory lock. Can be \"LOCKED\" "
            "or \"UNLOCKED\""
        "@replyvalue <locking_pid> process id of process that is supposed to have the lock"
        "@replyvalue <persistence_error> error text, if the persistence is incomplete";
}

tcn00_Error DBMSrvCmdDbmShm_DbmShmInfo::run(
    VControlDataT* vcontrol,
    CommandT* command,
    DBMSrv_Reply& theReply) {

    theReply.startWithOK();
    tcn00_Error returnValue(OK_CN00);
    DBMSrvMsg_Error errList;

    DBMSrvShM_Persistence* shmPers = DBMSrvShM_ShmAdmin::getShmPersistence();
    tcn002_XpValueString runDir;
    cn20XParamGetValue(
            cn00DBMServerData::vcontrol()->dbname,
            PAN_RUNDIR,
            runDir);
    shmPers->setRootDir(runDir.asCharp());
    
    bool shmExists(false);
    do {
        if( !shmPers->exists(shmExists, errList) ) {
            if( errList == DBMSrvMsg_Error::DBMSrv_SHMNODATAFILE ) {
                // info file exists, data file does not exist
                returnValue = examineShM(theReply);
                theReply.appendLine();
                theReply.appendLine(m_ChapterPers);
                theReply.appendLine(errList.Message());
            }
            else {
                returnValue = theReply.startWithMessageList(errList);
            }
            break;
        }
        if( shmExists ) {
            // shm exists completely
            returnValue = examineShM(theReply);
        }
        else {
            // shm does not exist at all
            char replyLine[1024];
            sprintf(
                replyLine,
                m_NoShM,
                shmPers->getInfoFile(),
                shmPers->getDataFile());
            theReply.appendLine(replyLine);
        }
        break;
    } while(true);
    delete shmPers;
    return returnValue;
}

tcn00_Error DBMSrvCmdDbmShm_DbmShmInfo::examineShM(DBMSrv_Reply& aReply) {
    // precondition: the info file exists
    char replyLine[1024];
    tcn00_Error returnValue(OK_CN00);
    
    // make sure to have a fresh shm instance
    DBMSrvShM_ShmAdmin::deleteInstance();
    do {
        // get the version
        char* vers(NULL);
        if( !DBMSrvShM_ShmAdmin::InfoInstance().isValid() ) {
            // can never happen (see precondition and preliminary checks)
            char buf[32];
            sprintf(buf, "%d", __LINE__ +2);
            Msg_List errShmPanic(DBMSrv_DBMError(SHMPANIC), 2, __FILE__, buf);
            returnValue = aReply.startWithMessageList( errShmPanic );
            break;
        }

        sprintf(replyLine, m_ChapterVers);
        aReply.appendLine(replyLine);
        vers = DBMSrvShM_ShmAdmin::InfoInstance().getRequiredVersion();
        sprintf( replyLine, m_ShmVersionRequired, vers );
        aReply.appendLine(replyLine);
        delete [] vers;

        vers = DBMSrvShM_ShmAdmin::InfoInstance().getActualVersion();
        sprintf( replyLine, m_ShmVersionFound, vers );
        aReply.appendLine(replyLine);
        delete [] vers;
        aReply.appendLine();

        // get the locking status
        sprintf(replyLine, m_ChapterLock);
        aReply.appendLine(replyLine);

        bool normalLock(false);
        bool backingLock(false);
        RTE_OSPid lockerPid(0);

        bool gotStatus(DBMSrvShM_ShmAdmin::InfoInstance().getLockStatus(
                    normalLock, backingLock, lockerPid));
        if( gotStatus ) {
            sprintf( replyLine, m_ShmLockNormal, normalLock?m_ShmLockedText:m_ShmUnlockedText);
            aReply.appendLine(replyLine);
            sprintf( replyLine, m_ShmLockBacking, backingLock?m_ShmLockedText:m_ShmUnlockedText);
            aReply.appendLine(replyLine);
            sprintf( replyLine, m_ShmLockingPid, lockerPid);
            aReply.appendLine(replyLine);
        }
        else {
            // report: unable to check lock status
        }
        aReply.appendLine();

        // get the participant pids
        sprintf(replyLine,m_ChapterProc);
        aReply.appendLine(replyLine);
        Tools_Vector<RTE_OSPid> vPid;
        DBMSrvShM_ShmAdmin::InfoInstance().getParticipantPIDsInto(vPid);
        Tools_Vector<RTE_OSPid>::iterator pid = NULL;

        // for formatting the output (length of pid)
        int maxPidLen(0);
        for( pid=vPid.begin(); pid!=vPid.end(); pid++ ) {
            RTE_OSPid aPid(*pid);
            int thisPidLen(0);
            while( aPid > 0 ) {
                aPid = aPid / 10;
                thisPidLen++;
            }
            if( thisPidLen > maxPidLen )
                maxPidLen = thisPidLen;
        }

        // create format string
        char format_line[32];
        char format_head[32];
        sprintf(format_head, m_ProcHeadFormat, maxPidLen);
        sprintf(format_line, m_ProcLineFormat, maxPidLen);

        sprintf( replyLine, format_head, m_PidText);
        aReply.appendLine(replyLine);

        for( pid=vPid.begin(); pid!=vPid.end(); pid++ ) {
            RTESys_ProcessState procState(*pid);
            const char* pText;
            switch( procState.State() ) {
                case RTESys_ProcessState::Alive :
                    pText = m_AliveText;
                    break;
                case RTESys_ProcessState::Died :
                    pText = m_DeadText;
                    break;
                default:
                    pText = m_UnknownText;
                    break;
            }
                sprintf( replyLine,
                    format_line,
                    *pid,
                    pText,
                    *pid==RTESys_ProcessID()?m_OwnPidMarker:"");
                aReply.appendLine(replyLine);
        }
        break;
    } while (true);
    DBMSrvShM_ShmAdmin::deleteInstance();
    return OK_CN00;
}

const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_AliveText = "ALIVE";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_DeadText = "DEAD";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_UnknownText = "UNKNOWN";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_CommandName = "dbm_shm_info";

const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ProcHeadFormat = "%%-%ds | status";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ProcLineFormat = "%%-%dd | %%s%%s";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_PidText = "pid";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_OwnPidMarker = " (*)";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ChapterProc = "PROCESSES";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ChapterLock = "LOCKING";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ChapterVers = "VERSION";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ChapterPers = "PERSISTENCE";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ShmLockedText = "LOCKED";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ShmUnlockedText = "UNLOCKED";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ShmLockNormal = "Normal lock  %s";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ShmLockBacking = "Backing lock %s";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ShmLockingPid = "Locking pid  %d";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ShmVersionRequired = "Required %s";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_ShmVersionFound = "Found    %s";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_NoDataFile = "data file missing";
const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_NoShM = "Shared memory does not exist (files %s and %s)";

==> maxdb-source-7_6_00_37.tgz: V76_00_37/MaxDB_ORG/sys/src/SAPDB/DBM/Srv/Commands/Dbm/Shm/DBMSrvCmdDbmShm_DbmShmInfo.cpp <==
/*!
  \file    DBMSrvCmdDbmShm_DbmShmInfo.cpp
  \author  MarcW
  \ingroup DBMServer shared memory administration commands
  \brief   list information about shared memory

    ========== licence begin  GPL
    Copyright (c) 2002-2005 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*/

#include "DBM/Srv/Commands/Dbm/Shm/DBMSrvCmdDbmShm_DbmShmInfo.hpp"
#include "DBM/Srv/SharedMemory/DBMSrvShM_ShmInfoReplier.hpp"

DBMSrvCmdDbmShm_DbmShmInfo::DBMSrvCmdDbmShm_DbmShmInfo()
: DBMSrv_Command( DBMSrv_Command::KeyDbmShmInfo, false )
{
    m_longHelp=
        "@command dbm_shm_info This command lists information about the DBM shared "
            "memory: version, locking status, connected processes and information about "
            "the persistence if necessary."
        "@preconditions You have the server authorization SharedMemoryAdmin."
        "@syntax dbm_shm_info"
        "@reply OK<NL>"
            "VERSION<NL>"
            "Required <req_version><NL>"
            "Found    <curr_version><NL>"
            "<NL>"
            "LOCKING<NL>"
            "Lock  <status_lock><NL>"
            "Reserve lock <status_reserve_lock><NL>"
            "Locking process <pid><NL>"
            "<NL>"
            "PROCESSES<NL>"
            "pid | status<NL>"
            "<pid_1> | <status_1><NL>"
            "...<NL>"
            "<pid_n> | <status_n><NL>"
            "<NL>"
            "[PERSISTENCE<NL>"
            "<error_message>]"
        "@replyvalue <pid_i> process id of a connected process"
        "@replyvalue <status_i> status of process <pid_i>. Status can be \"ALIVE\", \"DEAD\" "
            "or \"UNKNOWN\""
        "@replyvalue <req_version> required version of shared memory"
        "@replyvalue <curr_version> version of shared memory found on disk"
        "@replyvalue <status_lock> status of shared memory lock. Can be \"LOCKED\" "
            "or \"UNLOCKED\""
        "@replyvalue <status_reserve_lock> status of reserve shared memory lock. Can be \"LOCKED\" "
            "or \"UNLOCKED\""
        "@replyvalue <pid> process id of process that is supposed to have the lock"
        "@replyvalue <error_message> error text, if the persistence is incomplete";
}

tcn00_Error DBMSrvCmdDbmShm_DbmShmInfo::run(
    VControlDataT* vcontrol,
    CommandT* command,
    DBMSrv_Reply& theReply) {

    DBMSrvShM_ShmInfoReplier replier(vcontrol->dbname.asCharp(), theReply);
    return replier.replyInfo();
}

const char* const DBMSrvCmdDbmShm_DbmShmInfo::m_CommandName = "dbm_shm_info";
